@@ -35,9 +35,9 @@ import json as _json
 import random as _random
 
 
-__all__     = ['Matrix', 'MatrixError']
-__version__ = '0.6.1'
-__author__  = "Fares Ahmed <faresahmed@zohomail.com>"
+__all__ = ["Matrix", "MatrixError"]
+__version__ = "0.6.5"
+__author__ = "Fares Ahmed <faresahmed@zohomail.com>"
 
 
 class MatrixError(Exception):
@@ -53,9 +53,10 @@ class Matrix:
     def __init__(self, matrix):
         """Initialize Matrix Object | 3 Ways.
 
-        [Nested List]    Matrix([[1, 2, 3], [4, 5, 6]])
-        [One Number (I)] Matrix(3) -> (3x3) Identity Matrix
-        [String]         Matrix("1 2 3; 4 5 6")
+        Example:
+               [Nested List]  Matrix([[1, 2, 3], [4, 5, 6]])
+            [One Number (I)]  Matrix(3) -> (3x3) Identity Matrix
+                    [String]  Matrix("1 2 3; 4 5 6")
         """
         # Matrix([[1, 2, 3], [4, 5, 6]]) -> Row1 = (1 2 3), Row2 = (4 5 6)
         self.matrix = matrix
@@ -66,21 +67,22 @@ class Matrix:
 
         # Matrix("1 2 3; 4 5 6") -> Row1 = (1 2 3), Row2 = (4 5 6)
         if isinstance(matrix, str):
-            # input: Matrix("1 2 3; 4 5 6") | output: Matrix([[1, 2, 3], [4, 5, 6]])
-            matrix = matrix.split(';')             # ["1 2 3", " 4 5 6"]
-            matrix = list(map(str.lstrip, matrix)) # ["1 2 3", "4 5 6"]
-            for i, nums in enumerate(matrix):      # [['1', '2', '3'], ['4', '5', '6']]
-                matrix[i] = nums.split(' ')
+            # input: Matrix("1 2 3; 4 5 6")
+            # output: Matrix([[1, 2, 3], [4, 5, 6]])
+            matrix = matrix.split(";")  # ["1 2 3", " 4 5 6"]
+            matrix = list(map(str.lstrip, matrix))  # ["1 2 3", "4 5 6"]
+            for i, nums in enumerate(matrix):  # [['1', '2', '3'], ['4', '5', '6']]
+                matrix[i] = nums.split(" ")
 
             # list From str -> int
-            self.matrix = [list(map(int, matrix[i]))
-            for i in range(len(matrix))]
+            self.matrix = [list(map(int, matrix[i])) for i in range(len(matrix))]
 
         self.rowsnum = len(self.matrix)
         self.colsnum = len(self.matrix[0])
 
     def __repr__(self):
-        """Returnt a representation of the Matrix Object
+        """Returns a representation of the Matrix
+
         Appears when using an interactive Python shell
 
         >>> Matrix(3)
@@ -88,8 +90,7 @@ class Matrix:
         """
         result = list()
 
-        ma_str = [list(map(str, self.matrix[i]))
-        for i in range(self.rowsnum)]
+        ma_str = [list(map(str, self.matrix[i])) for i in range(self.rowsnum)]
 
         for i in ma_str:
             result.append(" ".join(i))
@@ -98,6 +99,7 @@ class Matrix:
 
     def __str__(self):
         """Return the matrix in `str` representation
+
         Appears when using print(Matrix)
 
         >>> print(Matrix.random((3,3), 1, 1000))
@@ -118,27 +120,30 @@ class Matrix:
             if len(max(row, key=len)) > maxlen:
                 maxlen = len(max(row, key=len))
 
-        for i in range(len(matrix_str)):
-            for x in range(len(matrix_str[0])):
-                rows += matrix_str[i][x] + " "
-                rows += " " * (maxlen-len(matrix_str[i][x]))
+        for rowsidx, row in enumerate(matrix_str):
+            for numidx, num in enumerate(matrix_str[0]):
+                rows += matrix_str[rowsidx][numidx] + " "
+                rows += " " * (maxlen - len(matrix_str[rowsidx][numidx]))
             rows = rows.rstrip()
             rows += "\n"
 
         # Calculate the spaces before (ROWSNUMxCOLSNUM)
-        rwcl_spaces = " " * (len(rows.split("\n")[-2]) // 2 -
-                            len(f"({self.rowsnum}x{self.colsnum})") // 2)
+        rwcl_spaces = " " * (
+            len(rows.split("\n")[-2]) // 2
+            - len(f"({self.rowsnum}x{self.colsnum})") // 2
+        )
 
         return rows + rwcl_spaces + f"({self.rowsnum}x{self.colsnum})"
 
     def __getitem__(self, rowcol):
-        """Return row, col, or item of Matrix Object
-        MatrixObject = Matrix("1 2 3; 4 5 6")
-
-        [Row] MatrixObject[1]  -> '4 5 6'
-        [Col] MatrixObject[:1] -> '2; 5'
-        [Item] MatrixObject[1, 2] -> 6
+        """Return row, col, or item of MatrixObject
 
+        Example:
+            MatrixObject = Matrix("1 2 3; 4 5 6")
+            [Row] MatrixObject[1]  -> '4 5 6'
+            [Col] MatrixObject[:1] -> '2; 5'
+            [Item] MatrixObject[1, 2] -> 6
+        
         Note: in [Item] the first arg is the row
         num and the second is the col num.
         """
@@ -154,7 +159,7 @@ class Matrix:
         return self.matrix[rowcol[0]][rowcol[1]]
 
     def __contains__(self, item):
-        """If item `in` MatrixObject | True if in the Matrix else False"""
+        """item in MatrixObject | True if in the Matrix else False"""
         for row in self.matrix:
             if item in row:
                 return True
@@ -187,10 +192,11 @@ class Matrix:
             # MatA + MatB
             result = list()
 
-            if (self.rowsnum != other.rowsnum or
-                self.colsnum != other.colsnum):
-                raise MatrixError('To add matrices, the matrices must have'
-                ' the same dimensions') from None
+            if self.rowsnum != other.rowsnum or self.colsnum != other.colsnum:
+                raise MatrixError(
+                    "To add matrices, the matrices must have"
+                    " the same dimensions"
+                ) from None
 
             for m in range(self.rowsnum):
                 result.append([])
@@ -214,10 +220,11 @@ class Matrix:
             # MatA + MatB
             result = list()
 
-            if (self.rowsnum != other.rowsnum or
-                self.colsnum != other.colsnum):
-                raise MatrixError('To sub matrices, the matrices must have'
-                ' the same dimensions') from None
+            if self.rowsnum != other.rowsnum or self.colsnum != other.colsnum:
+                raise MatrixError(
+                    "To sub matrices, the matrices must have"
+                    " the same dimensions"
+                ) from None
 
             for m in range(self.rowsnum):
                 result.append([])
@@ -239,14 +246,20 @@ class Matrix:
         if isinstance(other, Matrix):
             # MatA * MatB
             if self.colsnum != other.rowsnum:
-                raise MatrixError('The number of Columns in MatA must be'
-                ' equal to the number of Rows in MatB') from None
+                raise MatrixError(
+                    "The number of Columns in MatA must be"
+                    " equal to the number of Rows in MatB"
+                ) from None
 
             # References:
             # https://www.geeksforgeeks.org/python-program-multiply-two-matrices
-            result = [[sum(a * b for a, b in zip(A_row, B_col))
-                            for B_col in zip(*other.matrix)]
-                                    for A_row in self.matrix]
+            result = [
+                [
+                    sum(a * b for a, b in zip(A_row, B_col))
+                    for B_col in zip(*other.matrix)
+                ]
+                for A_row in self.matrix
+            ]
         else:
             # MatA * INT
             result = list()
@@ -260,39 +273,55 @@ class Matrix:
     # Object Math opertaions: END
 
     # Object Manpulation: START
-    def row(self, num: int, start=0):
-        """Return the row of the position `num`
-        Alternative to `MatrixObject[INT]` (it's
-        actullay what `MatrixObject[INT]` is using)
+    def row(self, position: int, start=0):
+        """Return the row in the position `position`
 
-        start=1 if you want to use the Matrix like
-        in real life.
+        Using `indexing` method is recommended
+        Matrix(3)[INT] == Matrix(3).row(INT)
+
+        Args:
+            position (int): the wanted row position
+            start (int, optional): start counting from. Defaults to 0.
+
+        Raises:
+            MatrixError: raised if the given position row
+            not exist in the Matrix
+
+        Returns:
+            Matrix: return the row as Matrix
         """
-        if num > start-1:
-            num -= start
+        if position > start - 1:
+            position -= start
 
         try:
-            return Matrix([self.matrix[num]])
+            return Matrix([self.matrix[position]])
         except IndexError:
-            raise MatrixError('Matrix Index out of range') from None
+            raise MatrixError("Matrix Index out of range") from None
 
+    def col(self, position: int, start=0):
+        """Return the col in the position `position`
 
-    def col(self, num: int, start=0):
-        """Return the col in the position `num`
-        Alternative to `MatrixObject[:INT]` (it's
-        actullay what `MatrixObject[:INT]` is using)
+        Using `indexing` method is recommended
+        Matrix(3)[:INT] == Matrix(3).col(INT)
 
-        start=1 if you want to use the Matrix Object
-        like in real life
+        Args:
+            position (int): the wanted column position
+            start (int, optional): start counting from. Defaults to 0.
+
+        Raises:
+            MatrixError: raised if the given position column
+            not exist in the Matrix
+
+        Returns:
+            Matrix: return the column as Matrix
         """
-        if num > start-1:
-            num -= start
+        if position > start - 1:
+            position -= start
 
         try:
-            return Matrix([[row[num]] for row in self.matrix])
+            return Matrix([[row[position]] for row in self.matrix])
         except IndexError:
-            raise MatrixError('Matrix Index out of range') from None
-
+            raise MatrixError("Matrix Index out of range") from None
 
     def addrow(self, row, index=-1):
         """Add a new row to your Matrix Object
@@ -313,7 +342,6 @@ class Matrix:
 
         return Matrix(result)
 
-
     def addcol(self, col, index=-1):
         """Add a new col to your Matrix Object
         MatrixObject -> '1 2 3; 5 6 7'
@@ -335,7 +363,6 @@ class Matrix:
 
         return Matrix(result)
 
-
     def rmrow(self, index):
         """Remove an Existing row from your Matrix Object.
         MatrixObject -> '1 2 3; 4 5 6'
@@ -351,11 +378,10 @@ class Matrix:
         try:
             result.pop(index)
         except IndexError:
-            raise MatrixError("Matrix index out of range") from None
+            raise MatrixError("Matrix Index out of range") from None
 
         return Matrix(result)
 
-
     def rmcol(self, index):
         """Remove an Existing col from your Matrix Object.
         MatrixObject -> '1 2 3 4; 5 6 7 8'
@@ -372,31 +398,76 @@ class Matrix:
             for i in range(self.rowsnum):
                 result[i].pop(index)
         except IndexError:
-            raise MatrixError("Matrix index out of range") from None
+            raise MatrixError("Matrix Index out of range") from None
 
         return Matrix(result)
 
+    def transpose(self):
+        """Swith the row and column indices of the Matrix
 
-    def transpose(self: list):
-        """Return the Matrix transposed
-        rows -> cols, cols -> rows
+        Returns:
+            Matrix: Swith the row and column indices of the Matrix
+            take a look at:
+            https://www.wikiwand.com/en/Transpose
+
+        Example:
+            >>> MatA = Matrix("1 2; 3 4; 5 6")
+            >>> print(MatA)
+            1 2
+            3 4
+            5 6
+            (3x2)
+            >>> print(MatA.transpose())
+            1 2 3
+            4 5 6
+            (2x3)
         """
         return Matrix([list(i) for i in zip(*self.matrix)])
 
-
     def tolist(self):
-        """Convert Matrix Object to a Nested List"""
+        """Convert Matrix Object to a Nested List
+
+        Returns:
+            list: Convert Matrix Object to a 
+            Nested List. Useful for manipulating
+            the list yourself if matrix-py doesn't
+            do what you expect.
+
+        Example:
+            >>> MatA = Matrix(3) # (3x3) Identity Matrix
+            >>> MatA.tolist()
+            [[1, 0, 0], [0, 1, 0], [0, 0, 1]]
+        """
         return self.matrix
     # Object Manpulation: END
 
     # Booleon Expressions: START
     def is_square(self):
-        """True if the matrix is square else False"""
+        """Return True if Matrix is square
+
+        Returns:
+            bool: True if the given Matrix (self)
+            is square (rows number == columns number).
+            else returns False
+        """
         return bool(self.rowsnum == self.colsnum)
 
-
     def is_symmetric(self):
-        """True if the matrix is symmetric else False"""
+        """Return True if Matrix is symmetric
+
+        Raises:
+            MatrixError: if the given Matrix is not
+            square Matrix. Explaination:
+            https://www.wikiwand.com/en/Symmetric_matrix 
+
+        Returns:
+            bool: True if the given Matrix (self)
+            is symmetric (Matrix == Matrix transpose).
+            else returns False
+        """
+        if not Matrix(self.matrix).is_square():
+            raise MatrixError("symmetric matrix is a square matrix")
+
         if self.matrix == (Matrix(self.matrix).transpose()).tolist():
             return True
         return False
@@ -405,11 +476,21 @@ class Matrix:
     # Pre Made Objects: START
     @staticmethod
     def identity(size: int):
-        """Return a New Identity Matrix (I)
-        Alternative to `Matrix(INT)`
+        """Return a new I (sizeXsize) Matrix
 
-        I Matrix is always square that's why
-        there's one arg
+        Args:
+            size (int): size=2 -> (2x2) Matrix
+
+        Returns:
+            Matrix: New I Matrix (All the diagonal
+            items == 1) 
+            
+        Example:
+            print(Matrix.identity(3)) # 3 -> (3x3) Matrix
+            Output: 1 0 0
+                    0 1 0
+                    0 0 1
+                    (3x3)
         """
         result = list()
 
@@ -419,21 +500,43 @@ class Matrix:
 
         return Matrix(result)
 
-
     @staticmethod
     def zero(size: int):
-        """Return a New Zero Matrix (All it's items = 0)"""
+        """Return a new zero (sizeXsize) Matrix
+
+        Args:
+            size (int): size=2 -> (2x2) Matrix
+
+        Returns:
+            Matrix: New zero Matrix (all matrix
+            items == 0)
+
+        Example:
+            print(Matrix.zero(3)) # 2 -> (2x2) Matrix
+            Output: 0 0 0
+                    0 0 0
+                    (3x3)
+        """
         return Matrix([[0] * size] * size)
 
-
     @staticmethod
-    def diagonal(*numbers: int):
-        """"Return a New diagonal Matrix OR get the
-        diagonal of YOUR MatrixObject.
-        MatrixObject = '1 2; 3 4'
+    def diagonal(*numbers: int, fill=0):
+        """Get the diag of a Matrix or Generate one
 
-        [New] Matrix.diagonal(1, 2, 3) -> '1 0 0; 0 2 0; 0 0 3'
-        [Get] Matrix.diagonal(MatrixObject) -> '1 4'
+        Returns:
+            Matrix: The result depends on how you call
+            the function. 
+            
+            Matrix.diagonal(MatrixObject):
+            Return the diagonal of an exisiting Matrix Object.
+            
+            Matrix.diagonal(INT, INT, ..):
+            Return a new square Matrix with (INT, INT, ..) as
+            the Matrix's diagonal
+
+            Matrix.diagonal(INT, fill=INT):
+            Return a new (INTxINT) Matrix with the diagonal
+            equels to the fill number
         """
         result = list()
 
@@ -441,27 +544,39 @@ class Matrix:
             result = numbers[0]
             return Matrix([[result[i, i] for i in range(result.rowsnum)]])
 
+        if fill:
+            for i in range(numbers[0]):
+                result.append([0] * numbers[0])
+                result[i][i] = fill
+
+            return Matrix(result)
+
         for i, number in enumerate(numbers):
             result.append([0] * len(numbers))
             result[i][i] = number
 
         return Matrix(result)
 
-
     @staticmethod
     def randint(size: tuple, a: int, b: int):
-        """Return random matrix in range [a, b]
-        Using randint from random module to generate
-        a MatrixObject (size) with random numbers
-        in range [a, b]
+        """Return (size) Matrix with random integers in range (a, b)
 
-        print(Matrix.randint((3,3), 1, 100))
-        Output: 1  25 58
-                59 18 48
-                15 6  70
-                  (3x3)
+        Args:
+            size (tuple): (3,3) -> (3x3) Matrix
+            a, b (int), (int): in the range a:b
+
+        Returns:
+            Matrix: (size) Matrix with random
+            integer in the range a:b
         """
-        result  = list()
+        if not isinstance(size, tuple):
+            raise TypeError("arg1 `size` must be tuple. (3,3) = (3x3) Matrix")
+        if not isinstance(a, int): 
+            raise TypeError("arg2 `a` must be int.")
+        if not isinstance(b, int): 
+            raise TypeError("arg2 `b` must be int.")
+        
+        result = list()
         rowsnum = size[0]
         colsnum = size[1]
 
@@ -477,64 +592,72 @@ class Matrix:
 def _cli():
     """The Command-Line Interface (CLI) for the module"""
 
-    parser=_argparse.ArgumentParser(
+    parser = _argparse.ArgumentParser(
         prog="matrix-py",
-        description = 'matrix-py module to add, substract, multiply'
-        'matrices.',
-        epilog = 'Usage: .. -ma "[[1, 2, 3], [4, 5, 6]]" -op "+" -mb'
-        ' "[[7, 8, 9], [10, 11, 12]]"')
+        description="matrix-py module to add, substract, multiply" "matrices.",
+        epilog='Usage: .. -ma "[[1, 2, 3], [4, 5, 6]]" -op "+" -mb'
+        ' "[[7, 8, 9], [10, 11, 12]]"',
+    )
 
-    parser.add_argument('-v', '--version',
+    parser.add_argument(
+        "-v",
+        "--version",
         action="version",
         version=__version__,
     )
 
-    parser.add_argument('-s', '--size',
-        type=_json.loads,
-        metavar='',
-        help='Size of MatA'
+    parser.add_argument(
+        "-s", "--size", type=_json.loads, metavar="", help="Size of MatA"
     )
 
-    parser.add_argument('-t', '--transpose',
+    parser.add_argument(
+        "-t",
+        "--transpose",
         type=_json.loads,
-        metavar='',
-        help='Transpose of MatA (-t "[[1, 2, 3], [4, 5, 6]]")'
+        metavar="",
+        help='Transpose of MatA (-t "[[1, 2, 3], [4, 5, 6]]")',
     )
 
-    parser.add_argument('-ma', '--matrixa',
+    parser.add_argument(
+        "-ma",
+        "--matrixa",
         type=_json.loads,
-        metavar='',
-        help='MatA (.. -ma "[[1, 2, 3], [4, 5, 6]]")'
+        metavar="",
+        help='MatA (.. -ma "[[1, 2, 3], [4, 5, 6]]")',
     )
 
-    parser.add_argument('-op', '--operator',
+    parser.add_argument(
+        "-op",
+        "--operator",
         type=str,
-        metavar='',
-        help='Operator (.. -op "+", "-", "*")'
+        metavar="",
+        help='Operator (.. -op "+", "-", "*")',
     )
 
-    parser.add_argument('-mb', '--matrixb',
+    parser.add_argument(
+        "-mb",
+        "--matrixb",
         type=_json.loads,
-        metavar='',
-        help='MatB (.. -mb "[[1, 2, 3], [4, 5, 6]]")'
+        metavar="",
+        help='MatB (.. -mb "[[1, 2, 3], [4, 5, 6]]")',
     )
 
-    parser.add_argument('-I', '--identity',
-        type=int,
-        metavar='',
-        help='Identity (.. -I 3)'
+    parser.add_argument(
+        "-I", "--identity", type=int, metavar="",
+        help="Identity (.. -I 3)"
     )
 
-    parser.add_argument('-i', '--int',
-        type=int,
-        metavar='',
-        help='Integer (.. -i 5)'
+    parser.add_argument(
+        "-i", "--int", type=int, metavar="",
+        help="Integer (.. -i 5)"
     )
 
-    parser.add_argument('-diag', '--diagonal',
+    parser.add_argument(
+        "-diag",
+        "--diagonal",
         type=_json.loads,
-        metavar='',
-        help='Diagonal (.. -diag [1, 2, 3, 4])'
+        metavar="",
+        help="Diagonal (.. -diag [1, 2, 3, 4])",
     )
 
     args = parser.parse_args()
@@ -555,17 +678,17 @@ def _cli():
         elif args.diagonal:
             b = Matrix.diagonal(args.diagonal)
 
-        if args.operator == '+':
+        if args.operator == "+":
             print(Matrix(args.matrixa) + b)
-        elif args.operator == '-':
+        elif args.operator == "-":
             print(Matrix(args.matrixa) - b)
-        elif args.operator == '*':
+        elif args.operator == "*":
             print(Matrix(args.matrixa) * b)
         else:
-            raise SyntaxError('The avillable operations are +, -, *')
+            raise SyntaxError("The avillable operations are +, -, *")
     else:
         print(parser.print_help())
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     _cli()
